@@ -22,7 +22,9 @@
 #include <ucommon/string.h>
 #include <ucommon/datetime.h>
 #include <ucommon/thread.h>
+#include <ucommon/timers.h>
 #include <stdlib.h>
+#include <ctype.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -35,9 +37,9 @@ using std::tm;
 using std::localtime;
 #endif
 
-const long DateTime::c_day = 86400l;
-const long DateTime::c_hour = 3600l;
-const long DateTime::c_week = 604800l;
+const long Time::c_day = 86400l;
+const long Time::c_hour = 3600l;
+const long Time::c_week = 604800l;
 
 const size_t Date::sz_string = 11;
 const size_t Time::sz_string = 9;
@@ -45,7 +47,7 @@ const size_t DateTime::sz_string = 20;
 
 #ifdef  HAVE_LOCALTIME_R
 
-tm_t *DateTime::glt(time_t *now)
+tm_t *DateTime::local(time_t *now)
 {
     tm_t *result, *dt = new tm_t;
     time_t tmp;
@@ -88,7 +90,7 @@ void DateTime::release(tm_t *dt)
 #else
 static mutex_t lockflag;
 
-tm_t *DateTime::glt(time_t *now)
+tm_t *DateTime::local(time_t *now)
 {
     tm_t *dt;
     time_t tmp;
@@ -144,13 +146,13 @@ Date::Date(const Date& copy)
 
 Date::Date(tm_t *dt)
 {
-    toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+    set(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
 }
 
 Date::Date(time_t tm)
 {
-    tm_t *dt = DateTime::glt(&tm);
-    toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+    tm_t *dt = DateTime::local(&tm);
+    set(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
     DateTime::release(dt);
 }
 
@@ -165,15 +167,15 @@ Date::~Date()
 
 void Date::set()
 {
-    tm_t *dt = DateTime::glt();
+    tm_t *dt = DateTime::local();
 
-    toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+    set(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
     DateTime::release(dt);
 }
 
 void Date::set(const char *str, size_t size)
 {
-    tm_t *dt = DateTime::glt();
+    tm_t *dt = DateTime::local();
     int nyear = 0;
     const char *mstr = str;
     const char *dstr = str;
@@ -229,37 +231,31 @@ void Date::set(const char *str, size_t size)
     DateTime::release(dt);
     ZNumber nmonth((char*)mstr, 2);
     ZNumber nday((char*)dstr, 2);
-    toJulian(nyear, nmonth(), nday());
+    set(nyear, nmonth(), nday());
 }
 
 Date::Date(int nyear, unsigned nmonth, unsigned nday)
 {
-    toJulian(nyear, nmonth, nday);
+    set(nyear, nmonth, nday);
 }
 
 void Date::update(void)
 {
 }
 
-bool Date::isValid(void) const
+bool Date::is_valid(void) const
 {
     if(julian == 0x7fffffffl)
         return false;
     return true;
 }
 
-char *Date::get(char *buf) const
-{
-    fromJulian(buf);
-    return buf;
-}
-
-time_t Date::getTime(void) const
+time_t Date::timeref(void) const
 {
     char buf[11];
     tm_t dt;
     memset(&dt, 0, sizeof(tm_t));
-    fromJulian(buf);
+    put(buf);
     Number nyear(buf, 4);
     Number nmonth(buf + 5, 2);
     Number nday(buf + 8, 2);
@@ -271,46 +267,31 @@ time_t Date::getTime(void) const
     return mktime(&dt); // to fill in day of week etc.
 }
 
-int Date::operator[](index_t idx) const
-{
-    switch(idx) {
-    case year:
-        return getYear();
-    case month:
-        return getMonth();
-    case day:
-        return getDay();
-    case dow:
-        return getDayOfWeek();
-    }
-    return -1;
-}
-
-int Date::getYear(void) const
+int Date::year(void) const
 {
     char buf[11];
-    fromJulian(buf);
+    put(buf);
     Number num(buf, 4);
     return num();
 }
 
-unsigned Date::getMonth(void) const
+unsigned Date::month(void) const
 {
     char buf[11];
-    fromJulian(buf);
+    put(buf);
     Number num(buf + 5, 2);
     return num();
 }
 
-unsigned Date::getDay(void) const
+unsigned Date::day(void) const
 {
     char buf[11];
-    fromJulian(buf);
+    put(buf);
     Number num(buf + 8, 2);
     return num();
 }
 
-unsigned Date::getDayOfWeek(void) const
+unsigned Date::dow(void) const
 {
     return (unsigned)((julian + 1l) % 7l);
 }
@@ -319,7 +300,7 @@ String Date::operator()() const
 {
     char buf[11];
 
-    fromJulian(buf);
+    put(buf);
     String date(buf);
 
     return date;
@@ -328,7 +309,7 @@ String Date::operator()() const
 long Date::get(void) const
 {
     char buf[11];
-    fromJulian(buf);
+    put(buf);
     return atol(buf) * 10000 + atol(buf + 5) * 100 + atol(buf + 8);
 }
 
@@ -374,37 +355,37 @@ Date& Date::operator-=(long val)
     return *this;
 }
 
-bool Date::operator==(const Date &d)
+bool Date::operator==(const Date &d) const
 {
     return julian == d.julian;
 }
 
-bool Date::operator!=(const Date &d)
+bool Date::operator!=(const Date &d) const
 {
     return julian != d.julian;
 }
 
-bool Date::operator<(const Date &d)
+bool Date::operator<(const Date &d) const
 {
     return julian < d.julian;
 }
 
-bool Date::operator<=(const Date &d)
+bool Date::operator<=(const Date &d) const
 {
     return julian <= d.julian;
 }
 
-bool Date::operator>(const Date &d)
+bool Date::operator>(const Date &d) const
 {
     return julian > d.julian;
 }
 
-bool Date::operator>=(const Date &d)
+bool Date::operator>=(const Date &d) const
 {
     return julian >= d.julian;
 }
 
-void Date::toJulian(long nyear, long nmonth, long nday)
+void Date::set(long nyear, long nmonth, long nday)
 {
     julian = 0x7fffffffl;
 
@@ -426,7 +407,7 @@ Date& Date::operator=(const Date& date)
     return *this;
 }
 
-void Date::fromJulian(char *buffer) const
+const char *Date::put(char *buffer) const
 {
 // The following conversion algorithm is due to
 // Henry F. Fliegel and Thomas C. Van Flandern:
@@ -454,6 +435,7 @@ void Date::fromJulian(char *buffer) const
     nday = int(k);
 
     buffer[10] = '\0';
+    return buffer;
 }
 
 Time::Time()
@@ -468,24 +450,24 @@ Time::Time(const Time& copy)
 
 Time::Time(tm_t *dt)
 {
-    toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+    set(dt->tm_hour, dt->tm_min, dt->tm_sec);
 }
 
 Time::Time(time_t tm)
 {
-    tm_t *dt = DateTime::glt(&tm);
-    toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+    tm_t *dt = DateTime::local(&tm);
+    set(dt->tm_hour, dt->tm_min, dt->tm_sec);
     DateTime::release(dt);
 }
 
-Time::Time(char *str, size_t size)
+Time::Time(const char *str, size_t size)
 {
     set(str, size);
 }
 
 Time::Time(int nhour, int nminute, int nsecond)
 {
-    toSeconds(nhour, nminute, nsecond);
+    set(nhour, nminute, nsecond);
 }
 
 Time::~Time()
@@ -494,38 +476,19 @@ Time::~Time()
 
 void Time::set(void)
 {
-    tm_t *dt = DateTime::glt();
-    toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+    tm_t *dt = DateTime::local();
+    set(dt->tm_hour, dt->tm_min, dt->tm_sec);
     DateTime::release(dt);
 }
 
-bool Time::isValid(void) const
+bool Time::is_valid(void) const
 {
     if(seconds == -1)
         return false;
     return true;
 }
 
-char *Time::get(char *buf) const
-{
-    fromSeconds(buf);
-    return buf;
-}
-
-int Time::operator[](index_t idx) const
-{
-    switch(idx) {
-    case hour:
-        return getHour();
-    case minute:
-        return getMinute();
-    case second:
-        return getSecond();
-    }
-    return -1;
-}
-
-int Time::getHour(void) const
+int Time::hour(void) const
 {
     if(seconds == -1)
         return -1;
@@ -533,7 +496,7 @@ int Time::getHour(void) const
     return (int)(seconds / 3600l);
 }
 
-int Time::getMinute(void) const
+int Time::minute(void) const
 {
     if(seconds == -1)
         return -1;
@@ -541,7 +504,7 @@ int Time::getMinute(void) const
     return (int)((seconds / 60l) % 60l);
 }
 
-int Time::getSecond(void) const
+int Time::second(void) const
 {
     if(seconds == -1)
         return -1;
@@ -554,7 +517,7 @@ void Time::update(void)
     seconds = abs(seconds % DateTime::c_day);
 }
 
-void Time::set(char *str, size_t size)
+void Time::set(const char *str, size_t size)
 {
     int sec = 00;
 
@@ -567,7 +530,7 @@ void Time::set(char *str, size_t size)
     }
 //00:00:00
     else if (size == 8) {
-        ZNumber nsecond(str + 6, 2);
+        ZNumber nsecond((char *)(str + 6), 2);
         sec = nsecond();
     }
     else {
@@ -575,16 +538,16 @@ void Time::set(char *str, size_t size)
         return;
     }
 
-    ZNumber nhour(str, 2);
-    ZNumber nminute(str+3, 2);
-    toSeconds(nhour(), nminute(), sec);
+    ZNumber nhour((char *)str, 2);
+    ZNumber nminute((char *)(str + 3), 2);
+    set(nhour(), nminute(), sec);
 }
 
 String Time::operator()() const
 {
     char buf[7];
 
-    fromSeconds(buf);
+    put(buf);
     String strTime(buf);
 
     return strTime;
@@ -637,32 +600,32 @@ Time Time::operator-(long val)
     return result;
 }
 
-bool Time::operator==(const Time &t)
+bool Time::operator==(const Time &t) const
 {
     return seconds == t.seconds;
 }
 
-bool Time::operator!=(const Time &t)
+bool Time::operator!=(const Time &t) const
 {
     return seconds != t.seconds;
 }
 
-bool Time::operator<(const Time &t)
+bool Time::operator<(const Time &t) const
 {
     return seconds < t.seconds;
 }
 
-bool Time::operator<=(const Time &t)
+bool Time::operator<=(const Time &t) const
 {
     return seconds <= t.seconds;
 }
 
-bool Time::operator>(const Time &t)
+bool Time::operator>(const Time &t) const
 {
     return seconds > t.seconds;
 }
 
-bool Time::operator>=(const Time &t)
+bool Time::operator>=(const Time &t) const
 {
     return seconds >= t.seconds;
 }
@@ -675,7 +638,7 @@ long Time::operator-(const Time &t)
         return seconds - t.seconds;
 }
 
-void Time::toSeconds(int nhour, int nminute, int nsecond)
+void Time::set(int nhour, int nminute, int nsecond)
 {
     seconds = -1;
 
@@ -685,7 +648,7 @@ void Time::toSeconds(int nhour, int nminute, int nsecond)
     seconds = 3600 * nhour + 60 * nminute + nsecond;
 }
 
-void Time::fromSeconds(char *buffer) const
+const char *Time::put(char *buffer) const
 {
     ZNumber zhour(buffer, 2);
     buffer[2] = ':';
@@ -697,6 +660,7 @@ void Time::fromSeconds(char *buffer) const
     zminute = (seconds - (3600l * zhour())) / 60l;
     zsecond = seconds - (3600l * zhour()) - (60l * zminute());
     buffer[8] = '\0';
+    return buffer;
 }
 
 Time& Time::operator=(const Time& time)
@@ -707,9 +671,9 @@ Time& Time::operator=(const Time& time)
 
 DateTime::DateTime(time_t tm)
 {
-    tm_t *dt = DateTime::glt();
-    toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
-    toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
+    tm_t *dt = DateTime::local();
+    Date::set(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+    Time::set(dt->tm_hour, dt->tm_min, dt->tm_sec);
     DateTime::release(dt);
 }
 
@@ -764,14 +728,14 @@ DateTime::DateTime(const char *a_str, size_t size)
 
 DateTime::DateTime(int year, unsigned month, unsigned day,
            int hour, int minute, int second) :
-  Date(year, month, day), Time(hour, minute, second)
+    Date(year, month, day), Time(hour, minute, second)
 {}
 
 DateTime::DateTime() : Date(), Time()
 {
-    tm_t *dt = DateTime::glt();
-    toSeconds(dt->tm_hour, dt->tm_min, dt->tm_sec);
-    toJulian(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
+    tm_t *dt = DateTime::local();
+    Time::set(dt->tm_hour, dt->tm_min, dt->tm_sec);
+    Date::set(dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday);
     DateTime::release(dt);
 }
 
@@ -785,16 +749,16 @@ void DateTime::set()
     Time::set();
 }
 
-bool DateTime::isValid(void) const
+bool DateTime::is_valid(void) const
 {
-    return Date::isValid() && Time::isValid();
+    return Date::is_valid() && Time::is_valid();
 }
 
-char *DateTime::get(char *buf) const
+const char *DateTime::put(char *buf) const
 {
-    fromJulian(buf);
+    Date::put(buf);
     buf[10] = ' ';
-    fromSeconds(buf+11);
+    Time::put(buf+11);
     return buf;
 }
 
@@ -804,7 +768,7 @@ time_t DateTime::get(void) const
     tm_t dt;
     memset(&dt, 0, sizeof(dt));
 
-    fromJulian(buf);
+    Date::put(buf);
     ZNumber nyear(buf, 4);
     ZNumber nmonth(buf + 5, 2);
     ZNumber nday(buf + 8, 2);
@@ -812,7 +776,7 @@ time_t DateTime::get(void) const
     dt.tm_mon = nmonth() - 1;
     dt.tm_mday = nday();
 
-    fromSeconds(buf);
+    Time::put(buf);
     ZNumber nhour(buf, 2);
     ZNumber nminute(buf + 2, 2);
     ZNumber nsecond(buf + 4, 2);
@@ -824,27 +788,6 @@ time_t DateTime::get(void) const
     return mktime(&dt);
 }
 
-int DateTime::operator[](index_t idx) const
-{
-    switch(idx) {
-    case year:
-        return getYear();
-    case month:
-        return getMonth();
-    case day:
-        return getDay();
-    case dow:
-        return getDayOfWeek();
-    case hour:
-        return getHour();
-    case minute:
-        return getMinute();
-    case second:
-        return getSecond();
-    }
-    return -1;
-}
-
 DateTime& DateTime::operator=(const DateTime& datetime)
 {
     julian = datetime.julian;
@@ -873,17 +816,17 @@ void DateTime::update(void)
     Time::update();
 }
 
-bool DateTime::operator==(const DateTime &d)
+bool DateTime::operator==(const DateTime &d) const
 {
     return (julian == d.julian) && (seconds == d.seconds);
 }
 
-bool DateTime::operator!=(const DateTime &d)
+bool DateTime::operator!=(const DateTime &d) const
 {
     return (julian != d.julian) || (seconds != d.seconds);
 }
 
-bool DateTime::operator<(const DateTime &d)
+bool DateTime::operator<(const DateTime &d) const
 {
     if (julian != d.julian) {
         return (julian < d.julian);
@@ -893,7 +836,7 @@ bool DateTime::operator<(const DateTime &d)
     }
 }
 
-bool DateTime::operator<=(const DateTime &d)
+bool DateTime::operator<=(const DateTime &d) const
 {
     if (julian != d.julian) {
         return (julian < d.julian);
@@ -903,7 +846,7 @@ bool DateTime::operator<=(const DateTime &d)
     }
 }
 
-bool DateTime::operator>(const DateTime &d)
+bool DateTime::operator>(const DateTime &d) const
 {
     if (julian != d.julian) {
         return (julian > d.julian);
@@ -913,7 +856,7 @@ bool DateTime::operator>(const DateTime &d)
     }
 }
 
-bool DateTime::operator>=(const DateTime &d)
+bool DateTime::operator>=(const DateTime &d) const
 {
     if (julian != d.julian) {
         return (julian > d.julian);
@@ -925,7 +868,7 @@ bool DateTime::operator>=(const DateTime &d)
 
 bool DateTime::operator!() const
 {
-    return !(Date::isValid() && Time::isValid());
+    return !(Date::is_valid() && Time::is_valid());
 }
 
 
@@ -938,7 +881,7 @@ String DateTime::format(const char *text) const
     String retval;
 
     t = get();
-    tbp = glt(&t);
+    tbp = local(&t);
     last = ::strftime(buffer, 64, text, tbp);
     release(tbp);
 
@@ -997,7 +940,7 @@ DateNumber::~DateNumber()
 
 void DateNumber::update(void)
 {
-    fromJulian(buffer);
+    Date::put(buffer);
 }
 
 void DateNumber::set(void)
@@ -1060,13 +1003,13 @@ void DateTimeString::update(void)
     DateTime::update();
     switch(mode) {
     case BOTH:
-        DateTime::get(buffer);
+        DateTime::put(buffer);
         break;
     case DATE:
-        Date::get(buffer);
+        Date::put(buffer);
         break;
     case TIME:
-        Time::get(buffer);
+        Time::put(buffer);
     }
 }
 
@@ -1082,3 +1025,93 @@ void DateTimeString::set(void)
     update();
 }
 
+isotime::isotime(Time& time)
+{
+    t = &time;
+    pos = 0;
+    mode = TIME;
+    time.put(buf);
+}
+
+isotime::isotime(Date& date)
+{
+    d = &date;
+    pos = 0;
+    mode = DATE;
+    date.put(buf);
+}
+
+isotime::isotime(Date& date, Time& time)
+{
+    d = &date;
+    t = &time;
+    pos = 0;
+    mode = DATETIME;
+    date.put(buf);
+    buf[10] = ' ';
+    time.put(buf + 11);
+}
+
+const char *isotime::_print(void) const
+{
+    return buf;
+}
+
+int isotime::_input(int code)
+{
+    if(isdigit(buf[pos]) && isdigit(code)) {
+        buf[pos++] = code;
+        if(buf[pos] == 0) {
+            code = EOF;
+            goto final;
+        }
+        return 0;
+    }
+
+    if(code == buf[pos]) {
+        ++pos;
+        return 0;
+    }
+
+final:
+    buf[pos] = 0;
+
+    switch(mode) {
+    case DATE:
+        d->set(buf);
+        break;
+    case TIME:
+        t->set(buf);
+        break;
+    case DATETIME:
+        buf[10] = 0;
+        d->set(buf);
+        t->set(buf + 11);
+        break;
+    };
+
+    return code;
+}
+
+extern "C" {
+    long tzoffset(struct timezone *tz)
+    {
+        struct timeval now;
+        time_t t1, t2 = 0;
+        struct tm t;
+        
+        gettimeofday(&now, tz);
+        t1 = now.tv_sec;
+
+#ifdef  HAVE_GMTIME_R
+        gmtime_r(&t1, &t);
+#else
+        t = *gmtime(&t1);
+#endif
+
+        t.tm_isdst = 0;
+        t2 = mktime(&t);
+        return (time_t)difftime(t1, t2);
+    } 
+}
+
