@@ -24,6 +24,7 @@
 #include <errno.h>
 #include <string.h>
 #include <stdarg.h>
+#include <limits.h>
 
 #if _POSIX_PRIORITY_SCHEDULING > 0
 #include <sched.h>
@@ -45,7 +46,7 @@ struct mutex_entry
 {
     pthread_mutex_t mutex;
     struct mutex_entry *next;
-    void *pointer;
+    const void *pointer;
     unsigned count;
 };
 
@@ -54,7 +55,7 @@ class __LOCAL rwlock_entry : public ThreadLock
 public:
     rwlock_entry();
     rwlock_entry *next;
-    void *object;
+    const void *object;
     unsigned count;
 };
 
@@ -132,24 +133,24 @@ inline pthread_t pthread_self(void)
 
 #endif
 
-static unsigned hash_address(void *ptr, unsigned indexing)
+static unsigned hash_address(const void *ptr, unsigned indexing)
 {
     assert(ptr != NULL);
 
     unsigned key = 0;
     unsigned count = 0;
-    unsigned char *addr = (unsigned char *)(&ptr);
+    const unsigned char *addr = (unsigned char *)(&ptr);
 
     if(indexing < 2)
         return 0;
 
     // skip lead zeros if little endian...
-    while(count < sizeof(void *) && *addr == 0) {
+    while(count < sizeof(const void *) && *addr == 0) {
         ++count;
         ++addr;
     }
 
-    while(count++ < sizeof(void *) && *addr)
+    while(count++ < sizeof(const void *) && *addr)
         key = (key << 1) ^ *(addr++);
 
     return key % indexing;
@@ -185,7 +186,7 @@ void ConditionalAccess::limit_sharing(unsigned max)
     max_sharing = max;
 }
 
-void Conditional::gettimeout(timeout_t msec, struct timespec *ts)
+void Conditional::set(struct timespec *ts, timeout_t msec)
 {
     assert(ts != NULL);
 
@@ -215,39 +216,21 @@ Conditional()
     used = 0;
 }
 
-void Semaphore::Shlock(void)
+void Semaphore::_share(void)
 {
     wait();
 }
 
-void Semaphore::Unlock(void)
+void Semaphore::_unlock(void)
 {
     release();
 }
 
-unsigned Semaphore::getUsed(void)
-{
-    unsigned rtn;
-    lock();
-    rtn = used;
-    unlock();
-    return rtn;
-}
-
-unsigned Semaphore::getCount(void)
-{
-    unsigned rtn;
-    lock();
-    rtn = count;
-    unlock();
-    return rtn;
-}
-
 bool Semaphore::wait(timeout_t timeout)
 {
     bool result = true;
     struct timespec ts;
-    gettimeout(timeout, &ts);
+    Conditional::set(&ts, timeout);
 
     lock();
     while(used >= count && result) {
@@ -487,7 +470,7 @@ Conditional::~Conditional()
 bool Conditional::wait(timeout_t timeout)
 {
     struct timespec ts;
-    gettimeout(timeout, &ts);
+    set(&ts, timeout);
     return wait(&ts);
 }
 
@@ -664,7 +647,7 @@ ConditionalAccess::~ConditionalAccess()
 bool ConditionalAccess::waitSignal(timeout_t timeout)
 {
     struct timespec ts;
-    gettimeout(timeout, &ts);
+    set(&ts, timeout);
     return waitSignal(&ts);
 }
 
@@ -681,7 +664,7 @@ bool ConditionalAccess::waitBroadcast(struct timespec *ts)
 bool ConditionalAccess::waitBroadcast(timeout_t timeout)
 {
     struct timespec ts;
-    gettimeout(timeout, &ts);
+    set(&ts, timeout);
     return waitBroadcast(&ts);
 }
 
@@ -698,12 +681,12 @@ bool ConditionalAccess::waitSignal(struct timespec *ts)
 #endif
 
 // abstract class never runs...
-bool Thread::isRunning(void)
+bool Thread::is_active(void)
 {
     return false;
 }
 
-bool JoinableThread::isRunning(void)
+bool JoinableThread::is_active(void)
 {
 #ifdef  _MSWINDOWS_
     return (running != INVALID_HANDLE_VALUE) && !joining;
@@ -712,7 +695,7 @@ bool JoinableThread::isRunning(void)
 #endif
 }
 
-bool DetachedThread::isRunning(void)
+bool DetachedThread::is_active(void)
 {
     return active;
 }
@@ -769,30 +752,12 @@ Conditional()
     waiting = 0;
 }
 
-unsigned RecursiveMutex::getWaiting(void)
-{
-    unsigned count;
-    Conditional::lock();
-    count = waiting;
-    Conditional::unlock();
-    return count;
-}
-
-unsigned RecursiveMutex::getLocking(void)
-{
-    unsigned count;
-    Conditional::lock();
-    count = lockers;
-    Conditional::unlock();
-    return count;
-}
-
-void RecursiveMutex::Exlock(void)
+void RecursiveMutex::_lock(void)
 {
     lock();
 }
 
-void RecursiveMutex::Unlock(void)
+void RecursiveMutex::_unlock(void)
 {
     release();
 }
@@ -801,7 +766,7 @@ bool RecursiveMutex::lock(timeout_t timeout)
 {
     bool result = true;
     struct timespec ts;
-    gettimeout(timeout, &ts);
+    set(&ts, timeout);
 
     Conditional::lock();
     while(result && lockers) {
@@ -854,44 +819,17 @@ ConditionalAccess()
     writers = 0;
 }
 
-unsigned ThreadLock::getAccess(void)
-{
-    unsigned count;
-    lock();
-    count = sharing;
-    unlock();
-    return count;
-}
-
-unsigned ThreadLock::getModify(void)
-{
-    unsigned count;
-    lock();
-    count = writers;
-    unlock();
-    return count;
-}
-
-unsigned ThreadLock::getWaiting(void)
-{
-    unsigned count;
-    lock();
-    count = waiting + pending;
-    unlock();
-    return count;
-}
-
-void ThreadLock::Exlock(void)
+void ThreadLock::_lock(void)
 {
     modify();
 }
 
-void ThreadLock::Shlock(void)
+void ThreadLock::_share(void)
 {
     access();
 }
 
-void ThreadLock::Unlock(void)
+void ThreadLock::_unlock(void)
 {
     release();
 }
@@ -902,7 +840,7 @@ bool ThreadLock::modify(timeout_t timeout)
     struct timespec ts;
 
     if(timeout && timeout != Timer::inf)
-        gettimeout(timeout, &ts);
+        set(&ts, timeout);
 
     lock();
     while((writers || sharing) && rtn) {
@@ -933,7 +871,7 @@ bool ThreadLock::access(timeout_t timeout)
     bool rtn = true;
 
     if(timeout && timeout != Timer::inf)
-        gettimeout(timeout, &ts);
+        set(&ts, timeout);
 
     lock();
     while((writers || pending) && rtn) {
@@ -984,7 +922,7 @@ auto_protect::auto_protect()
     object = NULL;
 }
 
-auto_protect::auto_protect(void *obj)
+auto_protect::auto_protect(const void *obj)
 {
     object = obj;
     if(object)
@@ -1004,7 +942,7 @@ void auto_protect::release()
     }
 }
 
-void auto_protect::operator=(void *obj)
+void auto_protect::operator=(const void *obj)
 {
     if(obj == object)
         return;
@@ -1015,24 +953,24 @@ void auto_protect::operator=(void *obj)
         Mutex::protect(object);
 }
 
-Mutex::gaurd::gaurd()
+Mutex::guard::guard()
 {
     object = NULL;
 }
 
-Mutex::gaurd::gaurd(void *obj)
+Mutex::guard::guard(const void *obj)
 {
     object = obj;
     if(obj)
         Mutex::protect(object);
 }
 
-Mutex::gaurd::~gaurd()
+Mutex::guard::~guard()
 {
     release();
 }
 
-void Mutex::gaurd::set(void *obj)
+void Mutex::guard::set(const void *obj)
 {
     release();
     object = obj;
@@ -1040,7 +978,7 @@ void Mutex::gaurd::set(void *obj)
         Mutex::protect(object);
 }
 
-void Mutex::gaurd::release(void)
+void Mutex::guard::release(void)
 {
     if(object) {
         Mutex::release(object);
@@ -1078,12 +1016,12 @@ void ThreadLock::indexing(unsigned index)
     }
 }
 
-ThreadLock::gaurd_reader::gaurd_reader()
+ThreadLock::guard_reader::guard_reader()
 {
     object = NULL;
 }
 
-ThreadLock::gaurd_reader::gaurd_reader(void *obj)
+ThreadLock::guard_reader::guard_reader(const void *obj)
 {
     object = obj;
     if(obj)
@@ -1091,12 +1029,12 @@ ThreadLock::gaurd_reader::gaurd_reader(void *obj)
             object = NULL;
 }
 
-ThreadLock::gaurd_reader::~gaurd_reader()
+ThreadLock::guard_reader::~guard_reader()
 {
     release();
 }
 
-void ThreadLock::gaurd_reader::set(void *obj)
+void ThreadLock::guard_reader::set(const void *obj)
 {
     release();
     object = obj;
@@ -1105,7 +1043,7 @@ void ThreadLock::gaurd_reader::set(void *obj)
             object = NULL;
 }
 
-void ThreadLock::gaurd_reader::release(void)
+void ThreadLock::guard_reader::release(void)
 {
     if(object) {
         ThreadLock::release(object);
@@ -1113,12 +1051,12 @@ void ThreadLock::gaurd_reader::release(void)
     }
 }
 
-ThreadLock::gaurd_writer::gaurd_writer()
+ThreadLock::guard_writer::guard_writer()
 {
     object = NULL;
 }
 
-ThreadLock::gaurd_writer::gaurd_writer(void *obj)
+ThreadLock::guard_writer::guard_writer(const void *obj)
 {
     object = obj;
     if(obj)
@@ -1126,12 +1064,12 @@ ThreadLock::gaurd_writer::gaurd_writer(void *obj)
             object = NULL;
 }
 
-ThreadLock::gaurd_writer::~gaurd_writer()
+ThreadLock::guard_writer::~guard_writer()
 {
     release();
 }
 
-void ThreadLock::gaurd_writer::set(void *obj)
+void ThreadLock::guard_writer::set(const void *obj)
 {
     release();
     object = obj;
@@ -1140,7 +1078,7 @@ void ThreadLock::gaurd_writer::set(void *obj)
             object = NULL;
 }
 
-void ThreadLock::gaurd_writer::release(void)
+void ThreadLock::guard_writer::release(void)
 {
     if(object) {
         ThreadLock::release(object);
@@ -1148,7 +1086,7 @@ void ThreadLock::gaurd_writer::release(void)
     }
 }
 
-bool ThreadLock::reader(void *ptr, timeout_t timeout)
+bool ThreadLock::reader(const void *ptr, timeout_t timeout)
 {
     rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
     rwlock_entry *entry, *empty = NULL;
@@ -1185,7 +1123,7 @@ bool ThreadLock::reader(void *ptr, timeout_t timeout)
     return false;
 }
 
-bool ThreadLock::writer(void *ptr, timeout_t timeout)
+bool ThreadLock::writer(const void *ptr, timeout_t timeout)
 {
     rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
     rwlock_entry *entry, *empty = NULL;
@@ -1222,7 +1160,7 @@ bool ThreadLock::writer(void *ptr, timeout_t timeout)
     return false;
 }
 
-void Mutex::protect(void *ptr)
+void Mutex::protect(const void *ptr)
 {
     mutex_index *index = &mutex_table[hash_address(ptr, mutex_indexing)];
     mutex_entry *entry, *empty = NULL;
@@ -1257,7 +1195,7 @@ void Mutex::protect(void *ptr)
     pthread_mutex_lock(&entry->mutex);
 }
 
-void ThreadLock::release(void *ptr)
+void ThreadLock::release(const void *ptr)
 {
     rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
     rwlock_entry *entry;
@@ -1281,7 +1219,7 @@ void ThreadLock::release(void *ptr)
     index->release();
 }
 
-void Mutex::release(void *ptr)
+void Mutex::release(const void *ptr)
 {
     mutex_index *index = &mutex_table[hash_address(ptr, mutex_indexing)];
     mutex_entry *entry;
@@ -1306,12 +1244,12 @@ void Mutex::release(void *ptr)
     index->release();
 }
 
-void Mutex::Exlock(void)
+void Mutex::_lock(void)
 {
     pthread_mutex_lock(&mlock);
 }
 
-void Mutex::Unlock(void)
+void Mutex::_unlock(void)
 {
     pthread_mutex_unlock(&mlock);
 }
@@ -1482,7 +1420,7 @@ bool TimedEvent::sync(void)
     if(!timeout)
         return false;
 
-    Conditional::gettimeout(timeout, &ts);
+    Conditional::set(&ts, timeout);
 
     if(pthread_cond_timedwait(&cond, &mutex, &ts) == ETIMEDOUT)
         return false;
@@ -1560,46 +1498,16 @@ ConditionalLock::Context *ConditionalLock::getContext(void)
     return slot;
 }
 
-unsigned ConditionalLock::getReaders(void)
-{
-    unsigned count;
-
-    lock();
-    count = sharing;
-    unlock();
-    return count;
-}
-
-unsigned ConditionalLock::getWaiters(void)
-{
-    unsigned count;
-
-    lock();
-    count = pending + waiting;
-    unlock();
-    return count;
-}
-
-void ConditionalLock::Shlock(void)
+void ConditionalLock::_share(void)
 {
     access();
 }
 
-void ConditionalLock::Unlock(void)
+void ConditionalLock::_unlock(void)
 {
     release();
 }
 
-void ConditionalLock::Exclusive(void)
-{
-    exclusive();
-}
-
-void ConditionalLock::Share(void)
-{
-    share();
-}
-
 void ConditionalLock::modify(void)
 {
     Context *context;
