@@ -35,8 +35,8 @@ extern "C" {
     __EXPORT int cpr_setenv(const char *s, const char *v, int p);
 
     inline int setenv(const char *s, const char *v, int overwrite)
-        {return cpr_setenv(s, v, overwrite);};
-};
+        {return cpr_setenv(s, v, overwrite);}
+}
 
 #endif
 
